@@ -8,10 +8,9 @@ from typing import BinaryIO, List, Optional, Union
 import boto3
 import requests
 
-from .dataset import Dataset, DatasetFile, UploadInfo
-
 # from tqdm import tqdm
 
+from .dataset import Dataset, DatasetFile, UploadInfo
 
 HEYWHALE_SITE = os.getenv("HEYWHALE_HOST", "https://www.heywhale.com")
 HEYWHALE_DS_BUCKET = os.getenv("HEYWHALE_DS_BUCKET", "kesci")
@@ -180,19 +179,13 @@ def upload_file(
     except Exception as e:
         print(f"Error putting object '{object_key}' from bucket '{bucket_name}': {e}")
     new_dataset_files: List[DatasetFile] = []
-    found_the_same_file = False
     for file in get_dataset(id, token).files:
-        print("hello", _get_file_in_dataset(prefix, file.key))
         if _get_file_in_dataset(prefix, file.key) != path_in_dataset:
             new_dataset_files.append(file)
-        elif overwrite:
-            found_the_same_file = True
-            new_dataset_files.append(DatasetFile("", object_key, 0))
-        else:
+        elif not overwrite:
             print("file exists, skip uploading")
             return
-    if not found_the_same_file:
-        new_dataset_files.append(DatasetFile("", object_key, 0))
+    new_dataset_files.append(DatasetFile("", object_key, 0))
     _update_dataset(id, new_dataset_files, token)
 
 
@@ -238,14 +231,98 @@ def _init_cache(cache_dir: Optional[str | Path] = None) -> Path:
         cache_dir = Path(os.getenv("MW_CACHE_DIR", "~/.cache/mw"))
     if isinstance(cache_dir, str):
         cache_dir = Path(cache_dir)
-    if not cache_dir.exists():
-        print(f"init cache dir: {cache_dir}")
+    # if not cache_dir.exists():
+    #     print(f"init cache dir: {cache_dir}")
     cache_dir.expanduser().mkdir(parents=True, exist_ok=True)
     cache_dir.expanduser().joinpath("blobs").mkdir(exist_ok=True)
     cache_dir.expanduser().joinpath("datasets").mkdir(exist_ok=True)
     return cache_dir
 
 
+def _download_single_file(
+    download_url: str, id: str, filename: str, cache_dir: Path
+) -> Path:
+    response = requests.get(download_url, stream=True, timeout=10)
+    total_size = int(response.headers.get("content-length", 0))
+    chunk_size = 4096  # 4 KB
+    slk_path = (
+        Path(cache_dir)
+        .expanduser()
+        .joinpath("datasets")
+        .joinpath(id)
+        .joinpath("snapshots")
+        .joinpath("1")
+        .joinpath(filename)
+    )
+    if slk_path.exists():
+        return slk_path
+    # TODO: add file lock
+    file_path = (
+        Path(cache_dir).expanduser().joinpath("blobs").joinpath(filename + ".lock")
+    )
+    file_path.parent.mkdir(parents=True, exist_ok=True)
+    if file_path.exists():
+        # TODO: wait for the lock
+        pass
+    # print("begin to download")
+    m = hashlib.md5()
+    # with tqdm(
+    #     total=total_size, desc=file_path, unit="iB", unit_scale=True, unit_divisor=1024
+    # ) as pbar,
+    with open(file_path, "wb") as file:
+        for chunk in response.iter_content(chunk_size=chunk_size):
+            m.update(chunk)
+            file.write(chunk)
+            # print("update")
+            # pbar.update(len(chunk))
+    etag = m.hexdigest()
+    # print(f"etag {etag}")
+    blob_path = file_path.rename(file_path.parent.joinpath(etag))
+    slk_path.parent.mkdir(parents=True, exist_ok=True)
+    slk_path.symlink_to(blob_path)
+    return slk_path
+
+
+def download_dir(
+    id: str,
+    cache_dir: Optional[str | Path] = None,
+    token: Optional[str] = None,
+) -> Path:
+    """Download a directory from the dataset.
+
+    Args:
+        id (str): The dataset id.
+
+    Returns:
+        str: The path to the downloaded directory.
+    """
+    if token is None:
+        token = os.getenv("MW_TOKEN")
+        if not token:
+            raise ValueError(
+                "No token provided and 'MW_TOKEN' environment variable is not set."
+            )
+    cache_dir = _init_cache(cache_dir)
+    url = f"{HEYWHALE_SITE}/api/datasets/{id}/downloadUrl"
+    headers = {
+        "x-kesci-token": token,
+    }
+    response = requests.get(url, headers=headers, timeout=10)
+    if response.status_code == 200:
+        files = response.json().get("files")
+        for file in files:
+            download_url = file.get("Url")
+            _download_single_file(
+                download_url, id, file.get("Name") + file.get("Ext"), cache_dir
+            )
+    else:
+        print(response.text)
+        response.raise_for_status()
+    dir =Path(cache_dir).expanduser().joinpath("datasets").joinpath(id).joinpath("snapshots").joinpath("1")
+    return dir
+    
+
+
 def download_file(
     id: str,
     filename: str,
@@ -276,7 +353,7 @@ def download_file(
         "x-kesci-token": token,
     }
     response = requests.get(url, headers=headers, timeout=10)
-    download_url = None
+    download_url: str = ""
     if response.status_code == 200:
         files = response.json().get("files")
         for file in files:
@@ -288,38 +365,5 @@ def download_file(
     else:
         print(response.text)
         response.raise_for_status()
-    response = requests.get(download_url, stream=True, timeout=10)
-    total_size = int(response.headers.get("content-length", 0))
-    chunk_size = 4096  # 4 KB
-    slk_path = (
-        Path(cache_dir)
-        .expanduser()
-        .joinpath("datasets")
-        .joinpath(id)
-        .joinpath("snapshots")
-        .joinpath("1")
-        .joinpath(filename)
-    )
-    if slk_path.exists():
-        return slk_path
-    # TODO: add file lock
-    file_path = (
-        Path(cache_dir).expanduser().joinpath("blobs").joinpath(filename + ".lock")
-    )
-    file_path.parent.mkdir(parents=True, exist_ok=True)
-    if file_path.exists():
-        # TODO: wait for the lock
-        pass
-    print("begin to download")
-    m = hashlib.md5()
-    with open(file_path, "wb") as file:
-        for chunk in response.iter_content(chunk_size=chunk_size):
-            m.update(chunk)
-            file.write(chunk)
-            # bar.update(len(chunk))
-    etag = m.hexdigest()
-    print(f"etag {etag}")
-    blob_path = file_path.rename(file_path.parent.joinpath(etag))
-    slk_path.parent.mkdir(parents=True, exist_ok=True)
-    slk_path.symlink_to(blob_path)
+    slk_path = _download_single_file(download_url, id, filename, cache_dir)
     return slk_path
